/*
 * Copyright 2005 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

#pragma ident	"$Id: print-test.c,v 1.3 2005/11/10 17:50:58 njacobs Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <locale.h>
#include <libintl.h>
#include <fcntl.h>
#include <errno.h>
#include <papi.h>

static void
usage(char *program)
{
	char *name;

	if ((name = strrchr(program, '/')) == NULL)
		name = program;
	else
		name++;

	fprintf(stdout,
		gettext("Usage: %s [-d destination] [-T content-type] (files)\n"),
		name);
	exit(1);
}

static struct {
	char *mime_type;
	char *lp_type;
} type_map[] = {
	{ "plain/text", "simple" },
	{ "application/octet-stream", "raw" },
	{ "application/octet-stream", "any" },
	{ "application/postscript", "postscript" },
	{ "application/postscript", "ps" },
	{ "application/x-cif", "cif" },
	{ "application/x-dvi", "dvi" },
	{ "application/x-plot", "plot" },
	{ "application/x-ditroff", "troff" },
	{ "application/x-troff", "otroff" },
	{ "application/x-pr", "pr" },
	{ "application/x-fortran", "fortran" },
	{ "application/x-raster", "raster" },
	{ NULL, NULL}
};

static char *
lp_type_to_mime_type(char *lp_type)
{
	int i;

	if (lp_type == NULL)
		return ("application/octet-stream");

	for (i = 0; type_map[i].lp_type != NULL; i++)
		if (strcasecmp(type_map[i].lp_type, lp_type) == 0)
			return (type_map[i].mime_type);

	return (lp_type);
}

int
add_file(char *printer, int id, char *file, int dump)
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_attribute_t **list = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	papi_job_t job = NULL;
	papi_stream_t *stream = NULL;
	int fd;

	printf("add_file(%s, %d, %s, %d)\n", printer, id, file, dump);

	status = papiServiceCreate(&svc, NULL, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	if ((fd = open(file, O_RDONLY)) < 0) {
		fprintf(stderr, "open(%s): %s\n", file, strerror(errno));
		return(-1);
	}


	status = papiJobStreamAdd(svc, printer, id, &stream);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext("add: JobStreamAdd: %s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	/* shovel the shit */
	{
		char buf[BUFSIZ];
		int rc;

		while ((status == PAPI_OK) &&
		       ((rc = read(fd, buf, sizeof (buf))) > 0))
			status = papiJobStreamWrite(svc, stream, buf, rc);
		if (status != PAPI_OK) {
			fprintf(stderr,
				gettext("add: JobStreamWrite: %s: %s\n"),
				printer, papiStatusString(status));
			exit(1);
		}
	}

	close(fd);

	status = papiJobStreamClose(svc, stream, &job);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext("add: JobStreamClose: %s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	list = papiJobGetAttributeList(job);
	if (dump != 0) {
		char buf[10240];

		buf[0] = '\0';
		papiAttributeListToString(list, "\n\t", buf, sizeof (buf));
		printf("job attributes:\n\t%s\n", buf);
	}
	papiJobFree(job);
	papiServiceDestroy(svc);
}

int
main(int ac, char *av[])
{
	papi_status_t status;
	papi_service_t svc = NULL;
	papi_attribute_t **list = NULL;
	papi_encryption_t encryption = PAPI_ENCRYPT_NEVER;
	papi_job_t job = NULL;
	char *printer = NULL;
	int copy = 0;
	int silent = 0;
	int dump = 0;
	int validate = 0;
	int modify = -1;
	int32_t id = 0;
	int c;

	(void) setlocale(LC_ALL, "");
	(void) textdomain("SUNW_OST_OSCMD");

	while ((c = getopt(ac, av, "DT:d:st:E")) != EOF)
		switch (c) {
		case 'T':	/* type */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"document-format",
					lp_type_to_mime_type(optarg));
			break;
		case 'D':	
			dump++;
			break;
		case 'd':	/* destination */
			printer = optarg;
			break;
		case 's':	/* suppress message */
			silent = 1;
			break;
		case 't':	/* title */
			papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
					"job-name", optarg);
			break;
		case 'E':
			encryption = PAPI_ENCRYPT_REQUIRED;
			break;
		default:
			usage(av[0]);
		}

	if ((printer == NULL) &&
	    ((printer = getenv("PRINTER")) == NULL) &&
	    ((printer = getenv("LPDEST")) == NULL))
		printer = "lp";

	if (modify == -1) {
		papiAttributeListAddString(&list, PAPI_ATTR_EXCL,
				"document-format", "application/octet-stream");
	}

	status = papiServiceCreate(&svc, printer, NULL, NULL, NULL,
					encryption, NULL);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext(
			"Failed to contact service for %s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	status = papiJobCreate(svc, printer, list, NULL, &job);
	if (status != PAPI_OK) {
		fprintf(stderr, gettext("jobCreate: %s: %s\n"), printer,
			papiStatusString(status));
		exit(1);
	}

	papiAttributeListFree(list);

	list = papiJobGetAttributeList(job);
	papiAttributeListGetInteger(list, NULL, "job-id", &id);

	if (dump != 0) {
		char buf[10240];

		buf[0] = '\0';
		papiAttributeListToString(list, "\n\t", buf, sizeof (buf));
		printf("job attributes:\n\t%s\n", buf);
	}
	papiJobFree(job);
	papiServiceDestroy(svc);

	for (c = optind ; c < ac ; c++)
		add_file(printer, id, av[c], dump);

	exit(0);
}
